﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Abt.Controls.SciChart;
using Abt.Controls.SciChart.Model.DataSeries;
using Abt.Controls.SciChart.Visuals;
using Abt.Controls.SciChart.Visuals.Annotations;
using Abt.Controls.SciChart.Visuals.RenderableSeries;
using WpfApplication1.Annotations;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window,INotifyPropertyChanged
    {
        private ObservableCollection<IChartSeriesViewModel> _graphicsCollection;
        private AnnotationCollection _annotationCollection;
        private bool _isCheckedDisplay;
        private List<XyyDataSeries<double, double>> _dataSeries;
        private XyyDataSeries<double, double> fifthSeries;
        private bool _visibilityOfCharts;

        public ObservableCollection<IChartSeriesViewModel> GraphicsCollection
        {
            get { return _graphicsCollection; }
            set
            {
                if (_graphicsCollection != value)
                {
                    _graphicsCollection = value;
                    OnPropertyChanged("GraphicsCollection");
                }
            }
        }

        public AnnotationCollection AnnotationCollection
        {
            get { return _annotationCollection; }
            set
            {
                _annotationCollection = value;
                OnPropertyChanged("AnnotationCollection");
            }
        }


        public bool IsCheckedDisplay
        {
            get { return _isCheckedDisplay; }
            set
            {
                _isCheckedDisplay = value;
                DisplayChanged();
                OnPropertyChanged("IsCheckedDisplay");
            }
        }

        public bool VisibilityOfCharts
        {
            get { return _visibilityOfCharts; }
            set
            {
                if(_visibilityOfCharts==value)return;
                _visibilityOfCharts = value;
                ChangeVisibility(value);
                OnPropertyChanged();
            }
        }


        private void DisplayChanged()
        {
            if (_isCheckedDisplay)
            {
            }
            else
            {

            }

        }

        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
            GraphicsCollection = new ObservableCollection<IChartSeriesViewModel>();
            AnnotationCollection = new AnnotationCollection();
            InitializeData();
        }

        private void InitializeData()
        {
            var xValues = new List<double>() { 0.0, 30.5, 767.1, 1001.0, 1049.0, 1107.2, 1150.0, 1238.0, 1242.0, 1360.0, 1534.9, 1557.5 };
            var yvalues = new List<double>() { -1681.9677866419572, -1681.9677866419572, -1785.4901654837388, -1830.9557197910513, -1842.9234639275237, -1859.6120453786864, -1870.2832839003745, -1903.1851356950374, -1876.7363682457765, -1884.9877320551109, -1860.4071400656626, -1854.351488316719 };
            var yvalues1 = new List<double>() { -1701.9677866419572, -1701.9677866419572, -1805.4901654837388, -1850.9557197910513, -1862.9234639275237, -1879.6120453786864, -1890.2832839003745, -1923.1851356950374, -1896.7363682457765, -1904.9877320551109, -1880.4071400656626, -1874.351488316719 };
            var yvalues2 = new List<double>() { -1731.9677866419572, -1731.9677866419572, -1835.4901654837388, -1880.9557197910513, -1892.9234639275237, -1909.6120453786864, -1920.2832839003745, -1953.1851356950374, -1926.7363682457765, -1934.9877320551109, -1910.4071400656626, -1904.351488316719 };
            var yvalues3 = new List<double>() { -1781.9677866419572, -1781.9677866419572, -1885.4901654837388, -1930.9557197910513, -1942.9234639275237, -1959.6120453786864, -1970.2832839003745, -2003.1851356950374, -1976.7363682457765, -1984.9877320551109, -1960.4071400656626, -1954.351488316719 };
            var yvalues4 = new List<double>() { -1831.9677866419572, -1831.9677866419572, -1935.4901654837388, -1980.9557197910513, -1992.9234639275237, -2009.6120453786864, -2020.2832839003745, -2053.1851356950374, -2026.7363682457765, -2034.9877320551109, -2010.4071400656626, -2004.351488316719 };
            var yvalues5 = new List<double>() { -1731.9677866419572, -1731.9677866419572, -1835.4901654837388, -1880.9557197910513, -1892.9234639275237, -1909.6120453786864, -1920.2832839003745, -1953.1851356950374, -1926.7363682457765, -1934.9877320551109, -1910.4071400656626, -1904.351488316719 };
            _dataSeries = new List<XyyDataSeries<double, double>>();
            var firstSeries = new XyyDataSeries<double, double>();
            firstSeries.Append(xValues, yvalues, yvalues1);
            var secondSeries = new XyyDataSeries<double, double>();
            secondSeries.Append(xValues, yvalues1, yvalues2);
            var thirdSeries = new XyyDataSeries<double, double>();
            thirdSeries.Append(xValues, yvalues2, yvalues3);
            var forthSeries = new XyyDataSeries<double, double>();
            forthSeries.Append(xValues, yvalues3, yvalues4);
            fifthSeries = new XyyDataSeries<double, double>();
            fifthSeries.Append(xValues, yvalues4, yvalues4);
            _dataSeries.Add(firstSeries);
            _dataSeries.Add(secondSeries);
            _dataSeries.Add(thirdSeries);
            _dataSeries.Add(forthSeries);
//            _dataSeries.Add(fifthSeries);
            CreateRenderableSeries(_dataSeries);

        }

        private void ChangeVisibility(bool isVisible)
        {
            if (GraphicsCollection != null)
            {
                var renderSeries = GraphicsCollection.Select(q => q.RenderSeries).ToList();
                foreach (var series in renderSeries)
                {
                    if(series.PaletteProvider is TestPaletteProvider)
                        series.PaletteProvider=new TestPaletteProvider() {IsVisible = isVisible };
                    
                }
            }
        }


        private void CreateRenderableSeries(List<XyyDataSeries<double, double>> dataSeries)
        {
            for (int i = 0; i < dataSeries.Count; i++)
            {
                GraphicsCollection.Add(new ChartSeriesViewModel(dataSeries[i], CreateRenderableSeries()));
            }
        }

        public IRenderableSeries CreateRenderableSeries()
        {
            var renderableSeries = new FastLineRenderableSeries()
            {
                SeriesColor = Colors.Blue,
//                BandUpColor = PickColor(count),
                StrokeThickness = 1,
                Visibility = Visibility.Visible,
                AntiAliasing = false,
                ZeroLineY = Double.NegativeInfinity,
                PaletteProvider = new TestPaletteProvider()
            };
            return renderableSeries;
        }

        private Brush PickBrush(int number)
        {
            switch (number % 3)
            {
                case 0:
                    return Brushes.Red;
                case 1:
                    return Brushes.Blue;
                default:
                    return Brushes.GreenYellow;
            }
        }

        private Color PickColor(int number)
        {
            switch (number % 3)
            {
                case 0:
                    return Colors.Red;
                case 1:
                    return Colors.Blue;
                default:
                    return Colors.GreenYellow;
            }
        }


        private void LoadedData()
        {
            DataContext = this;
        }

        private void SciChartSurface_Loaded(object sender, RoutedEventArgs e)
        {
            LoadedData();
        }


        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SciChartSurface_OnMouseDown(object sender, MouseButtonEventArgs e)
        {
            //GraphicsCollection.Add(new ChartSeriesViewModel(fifthSeries, CreateRenderableSeries()));
            var firstDef=GraphicsCollection.FirstOrDefault();
          
            if (firstDef != null)
            {
                firstDef.RenderSeries.DataSeries.DataSeriesChanged += DataSeries_DataSeriesChanged;
                firstDef.DataSeries.YValues[2]= (double)firstDef.DataSeries.YValues[2]+20.0;
                ((BaseRenderableSeries)firstDef.RenderSeries).OnInvalidateParentSurface();
            }
            
        }

        private void DataSeries_DataSeriesChanged(object sender, DataSeriesChangedEventArgs e)
        {
            MessageBox.Show("Show");

        }

        private void SciChartSurface_OnPreviewMouseMove(object sender, MouseEventArgs e)
        {
            //Console.WriteLine(string.Format("{0} {1}",position.X,position.Y));
        }
    }
}
